﻿using System;
using System.Collections.Generic;
using System.Globalization;
using TSLab.DataSource;
using TSLab.Script;
using System.Linq;
using TSLab.Script.Handlers;

namespace RusAlgo.Handlers.Public
{
    /// <summary>
    /// Вспомогательный класс, содержащий хелпер методы. В том числе и методы расширения.
    /// </summary>
    public static class TradeHelper
    {
        #region Общие хелперы

        public static bool IsNull(this object obj)
        {
            return obj == null;
        }

        /// <summary>
        /// Метод упрощающий форматирование строк.
        /// </summary>
        /// <param name="str">Строка для форматирования.</param>
        /// <param name="args">Аргументы для форматирования.</param>
        /// <returns></returns>
        public static string Put(this string str, params object[] args)
        {
            // Вводим культуру для того чтобы double числа не переводились как числа с запятой! Это гадит.
            return string.Format(CultureInfo.InvariantCulture, str, args);
        }

        /// <summary>
        /// Выводит в лог информационное сообщение.
        /// </summary>
        /// <param name="ctx"></param>
        /// <param name="str"></param>
        /// <param name="args"></param>
        public static void LogInfo(this IContext ctx, string str, params object[] args)
        {
            // Задаем цвет соообщения
            var color = System.Drawing.Color.Blue;
            
            // Формируем строку, вставляем спец слово Info.
            var msg = string.Format("Info: " + str, args);
            
            ctx.Log(msg, new Color(color.ToArgb()));
        }

        /// <summary>
        /// Выводит в лог сообщение об ошибке.
        /// </summary>
        /// <param name="ctx"></param>
        /// <param name="str"></param>
        /// <param name="args"></param>
        public static void LogError(this IContext ctx, string str, params object[] args)
        {
            // Задаем цвет соообщения
            var color = System.Drawing.Color.Red;

            // Формируем строку, вставляем спец слово Info.
            var msg = string.Format("Error: " + str, args);

            ctx.Log(msg, new Color(color.ToArgb()));
        }

        #endregion

        #region Списки

        /// <summary>
        /// Копирует элементы списка в другой список. Копирует заданное число элементов начиная с заданного индекса.
        /// </summary>
        /// <typeparam name="T">Тип элементов списка.</typeparam>
        /// <param name="collection">Список элементов, не должен быть пустым и null</param>
        /// <param name="start">Индекс, начиная с которого копировать элементы. Начинается с 0.</param>
        /// <param name="count">Число элементов которые надо скопировать.</param>
        /// <param name="forward">Если истина, то элементы берутся от start в сторону конца коллекции. Иначе в сторону начала</param>
        /// <returns>Если start больше числа элементов коллекции то вернет пустой список. 
        /// Иначе будет копировать элементы в новый список пока не наберется count элементов. 
        /// Если коллекция закончится раньше, возвращает все что получилось скопировать.</returns>
        public static IList<T> CopyCountTo<T>(this IList<T> collection, int start, int count, bool forward = true)
        {
            // Проверка аргументов.
            if (!collection.AnyNotNull())
                throw new ArgumentException("Коллекция должна содержать элементы.", "collection");

            if (start > collection.Count - 1)
                return new T[0];

            if (count == 0)
                return new T[0];

            // Копируем заданные элементы для форвард.
            if (forward)
            {
                var copyCounter = 0;
                var valueCount = Math.Min(collection.Count - start, count);
                var values = new T[valueCount];
                for (var i = start; i < collection.Count; i++)
                {
                    values[copyCounter] = collection[i];
                    copyCounter++;

                    if (copyCounter >= valueCount)
                        break;
                }

                return values;
            }
            else   // копируем для обратного копирования
            {
                var valueCount = Math.Min(start+1, count);
                var values = new T[valueCount];

                var copyCounter = values.Length-1;
                for (var i = start; i >= 0; i--)
                {
                    values[copyCounter] = collection[i];
                    copyCounter--;

                    if (copyCounter < 0)
                        break;
                }

                return values;
            }
        }

        /// <summary>
        /// Проверяет наличие элементов в коллекции. Так же работает если null значение. Тоже вернет false
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="collection"></param>
        /// <returns></returns>
        public static bool AnyNotNull<T>(this IEnumerable<T> collection)
        {
            if (object.ReferenceEquals(collection, null))
                return false;

            return collection.Any();
        }

        /// <summary>
        /// Заполняет список заданным элементов. Все ячейки списка будут равны value
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="list">Список который заполнять</param>
        /// <param name="value">Значение которым заполнять</param>
        public static void Fill<T>(this IList<T> list, T value)
        {
            for (var i = 0; i < list.Count; i++)
            {
                list[i] = value;
            }
        }

        #endregion

        #region Базовые типы данных

        /// <summary>
        /// Определяет является ли число бесконечностью
        /// </summary>
        /// <param name="d"></param>
        /// <returns></returns>
        public static bool IsInfinity(this double d)
        {
            return double.IsInfinity(d);
        }

        /// <summary>
        /// Определяет является ли значение НЕ ЧИСЛОМ, то есть неопределенностью
        /// </summary>
        /// <param name="d"></param>
        /// <returns></returns>
        public static bool IsNaN(this double d)
        {
            return double.IsNaN(d);
        }

        /// <summary>
        /// Четное ли число?
        /// </summary>
        /// <param name="i"></param>
        /// <returns></returns>
        public static bool IsEven(this int i)
        {
            return i%2 == 0;
        }

        /// <summary>
        /// НЕчетное ли число?
        /// </summary>
        /// <param name="i"></param>
        /// <returns></returns>
        public static bool IsOdd(this int i)
        {
            return i%2 != 0;
        }

        #endregion
    }
}
